require_relative 'adapter/plm.rb'
require_relative 'user/plm.rb'
require_relative 'user/search.rb'
require_relative 'user/emath.rb'

module PLM
  module Model
    class User
      attr_reader :identity, :mails_profile
      #
      # Initialise les différentes informations du profile selon que je possède un compte PLM
      # que je soit dans ma communauté ou non, etc.
      def initialize(identity)
        @identity = identity
        if @identity.known?
          @mails_profile = @identity.mails if @identity.converged_from_insmi? || @identity.user?
        end
      end

      def plm
        if @plm.nil?
          #Plm::Adapter.type = :ldap
          #@plm = Plm.new(identity.user).user if identity.user?
          @plm = Plm.new(identity.user).adapter if identity.user?
        end
        @plm
      end

      def emath
        if get_type == 'insmi' || get_type == 'emath'
          #Emath::Adapter.type = :ldap
          @emath = Emath.new(mails_profile[:emath])
        end
      end

      def affectation
        plm.affectation
      end

      def is_root?
        plm.is_root?
      end

      def expired
        plm.expired
      end

      def profile
        h = {}
        h[:identity]=login_infos
        mails = get_mails
        h[:mails]=mails if mails
        keys = plm.get_sshKeys
        h[:sshkeys]=keys if keys
        if emath.persons
          h[:civility]=emath.get_civility
          h[:classification]=emath.get_classification
        end
        h
      end

      def get_entities
        h = []
        unless identity.emath_entities.nil?
          identity.emath_entities.each do |lab|
            data = JSON.parse(URI.parse('http://api.mathdoc.fr/labo/'+lab).read)
            unless (data['name'].nil?||data['alias'].nil?)
              h.push({:name=>data['name'],:alias=>data['alias']})
            end
          end
        end
        h
      end
      def get_type
        return 'other' if identity.other?
        return 'insmi' if identity.converged_from_insmi?
        return 'emath' if identity.user? && identity.converged_from_emath?
        return 'plm'   if identity.user?
      end
      def may_converge?
        identity.converged_from_insmi? || identity.other?
      end
      def may_converge
        return [{"type"=>"associate"},{"type"=>"new"}] if identity.converged_from_insmi?
        return [{"type"=>"associate"}] if identity.other?
        return [{"type"=>"converged"}] if identity.converged?
        return [{"type"=>"unknown"}]
      end
      #
      # Retourne un hash permettant de traiter les différentes types de profil
      #
      def login_infos
        {type:        get_type,
         user:        identity.user,
         converge:    may_converge,
         displayName: identity.names.fetch("displayName",'Name unknown'),
         SN:          identity.names.fetch("SN",'Name unknown'),
         givenName:   identity.names.fetch("givenName",'Name unknown'),
         mail:        identity.mail,
         from:        identity.idp,
         manage:      identity.admin,
         entities:    get_labs}
      end
      def get_mails
        h = []
        h.push({removable:false,type:'from',email:identity.mail})
        if identity.mails[:contact] &&
          identity.mails[:contact] != identity.mail
          h.push({removable:false,type:'contact',email:identity.mails[:contact]})
        end
        if identity.mails[:mathrice]
          h.push({removable:false,type:'plm',email:identity.mails[:mathrice]})
        end
        if identity.converged?
          if identity.mails[:altMathrice]
            identity.mails[:altMathrice].each do |m|
              removable = (m != identity.mail)
              type = (identity.mails[:emath].include?(m)) ? 'emath':'idp'
              h.push({removable:removable,type:type,email:m})
            end
          end
          identity.mails[:emath].each_with_index do |emath,i|
            unless emath == identity.mail && identity.mails[:altMathrice].include?(emath)
              type = i==0?'emath':'altEmath'
              h.push({removable:false,type:type,email:emath})
            end
          end
        end
        h.uniq!{ |s| s[:email] }
        h
      end

      def del_mail(req)
        plm.del_mail(req, identity.mail) if plm
      end

      def get_labs
        a = {}
        a[:emath] = get_entities
        a[:plm]   = plm.get_plm_labs if plm
        a
      end

      def associate(env)
        if (!env["HTTP_CAS_USER"].nil?&&
          (get_type == 'insmi' || get_type == 'other'))
          uid = env["HTTP_CAS_USER"]
          Plm.associate(uid, identity.mail)
        end
      end
    end
  end
end
